%% Version 2.0 BlOGS Code
%% Please refer to log_of_changes.txt for a list of changes over the previous version.
%% (c) 2011 University of South Florida, Tampa
%% All rights reserved
%% Use, or copying without permission prohibited.
%% This permission may be granted only by Prof. Sudeep Sarkar of University
%% of South Florida (sarkar@cse.usf.edu).  
%%
%%     This program is free software for research and educational purposes
%%     ONLY. For these purposes, you can redistribute it and/or modify it
%%     under the terms of the GNU General Public License as published by
%%     the Free Software Foundation, either version 3 of the License, or
%%     (at your option) any later version. This program is distributed in
%%     the hope that it will be useful, but WITHOUT ANY WARRANTY; without
%%     even the implied warranty of MERCHANTABILITY or FITNESS FOR A
%%     PARTICULAR PURPOSE.  See the GNU General Public License for more
%%     details. You should have received a copy of the GNU General Public
%%     License along with this program.  If not, see
%%     <http://www.gnu.org/licenses/>.
%%
%%
%%    BEFORE ANY RESULTS OR OUTPUT OF THIS PROGRAM ARE DISTRIBUTED TO THIRD
%%    PARTY, e.g. in publications, reports, etc., THEY HAVE TO BE SHARED
%%    WITH Prof. Sudeep Sarkar of University of South Florida (sarkar@cse.usf.edu). 
%%
%%    AUTHOR OF CODE: Aveek Brahmachari (abrahmac@mail.usf.edu)
%% 
%% Acknowledgment as appropriate is respectfully requested to 
%%
%% Brahmachari, A.S.; Sarkar, S.; , "BLOGS: Balanced local and global
%% search for non-degenerate two view epipolar geometry," Computer Vision,
%% 2009 IEEE 12th International Conference on , vol., no., pp.1685-1692,
%% Sept. 29 2009-Oct. 2 2009 doi: 10.1109/ICCV.2009.5459379

%% x11 - [3,N1] homogenous coordiantes of the N1 points in first images
%% x22 - [3,N2] homogenous coordiantes of the N2 points in second images
%% PS - normalized cumulative distribution for photometric sampling (global
%% search)
%% sz - size of the image 
%%      sz(1,1) - height of first image;
%%      sz(1,2) - width of first image;
%%      sz(2,1) - height of second image;
%%      sz(2,2) - width of second image;
%% paramBLOGS - other parameters of the BLOGS algorithm

%% optind - optimal index of minimal correspondences chosen to generate the
%% best fundamental matrix
%% maxsup - a soft estimate of the number of inlier correspondences
%% inlier_rate - a soft estimate of the inlier rate
%% F - fundamental matrix
%% 

function [optind,maxsup,inlier_rate,inliers,F] = BLOGS(x11, x22, PS, sz, paramBLOGS)

    maxsup = 0;
    
    %BLOGS parameters initialization
    alpha       = paramBLOGS.alpha;
    beta        = paramBLOGS.beta;
    tau         = paramBLOGS.tau;
    sample_size = paramBLOGS.sample_size;
    iter        = paramBLOGS.iter;

    ind = 1 : sample_size;

    [x1, x2] = points_normalize(x11, x22, sz);
    if(beta>0)
        [Q] = degeneracy_measure(x1, x2);
    end
    [W]     = info_tensor(x1(:,ind),x2(:,ind));
    [P]     = jfd_probability(x1,x2,W, tau);
    optind  = ind;
    inliers = ind;
        
    for n = 1 : iter
        if(sample_size == 7)
            [F, sup, prob] = psupportF7(x1,x2,ind,sz);
        else
            [F, sup, prob] = psupportF(x1,x2,ind,sz);
        end
        if(beta>0)
            [pMh] = NonDegeneracy(ind,Q, beta);
        else
            pMh = 1;
        end
        if (sup*pMh > maxsup)
            maxsup  = sup;
            optind  = ind;
            [W]     = info_tensor(x1(:,ind),x2(:,ind));
            [P]     = jfd_probability(x1,x2,W,tau);
            inliers = find(prob >= 0.95);
        end

        x = rand;
        if(x < alpha)
            ind = guidedrandsample(PS,sample_size);
        else
            ind = guidedrandsample(P,sample_size);
        end

    end

    inlier_rate = maxsup/size(x1,2);
end