%% Version 2.0 BlOGS Code
%% Please refer to log_of_changes.txt for a list of changes over the previous version.
%% (c) 2011 University of South Florida, Tampa
%% All rights reserved
%% Use, or copying without permission prohibited.
%% This permission may be granted only by Prof. Sudeep Sarkar of University
%% of South Florida (sarkar@cse.usf.edu).  
%%
%%     This program is free software for research and educational purposes
%%     ONLY. For these purposes, you can redistribute it and/or modify it
%%     under the terms of the GNU General Public License as published by
%%     the Free Software Foundation, either version 3 of the License, or
%%     (at your option) any later version. This program is distributed in
%%     the hope that it will be useful, but WITHOUT ANY WARRANTY; without
%%     even the implied warranty of MERCHANTABILITY or FITNESS FOR A
%%     PARTICULAR PURPOSE.  See the GNU General Public License for more
%%     details. You should have received a copy of the GNU General Public
%%     License along with this program.  If not, see
%%     <http://www.gnu.org/licenses/>.
%%
%%
%%    BEFORE ANY RESULTS OR OUTPUT OF THIS PROGRAM ARE DISTRIBUTED TO THIRD
%%    PARTY, e.g. in publications, reports, etc., THEY HAVE TO BE SHARED
%%    WITH Prof. Sudeep Sarkar of University of South Florida (sarkar@cse.usf.edu). 
%%
%%    AUTHOR OF CODE: Aveek Brahmachari (abrahmac@mail.usf.edu)
%% 
%% Acknowledgment as appropriate is respectfully requested to 
%%
%% Brahmachari, A.S.; Sarkar, S.; , "BLOGS: Balanced local and global
%% search for non-degenerate two view epipolar geometry," Computer Vision,
%% 2009 IEEE 12th International Conference on , vol., no., pp.1685-1692,
%% Sept. 29 2009-Oct. 2 2009 doi: 10.1109/ICCV.2009.5459379

function [info] =  corrinfo(varargin)

    [x1, x2, npts] = checkargs(varargin(:));
    v = version; Octave = v(1)<'5';  % Crude Octave test
    
    A = [x2(1,:)'.*x1(1,:)'   x2(1,:)'.*x1(2,:)'  x2(1,:)' ...
         x2(2,:)'.*x1(1,:)'   x2(2,:)'.*x1(2,:)'  x2(2,:)' ...
         x1(1,:)'             x1(2,:)'            ones(npts,1) ]; 

    if Octave
	[U,D,V] = svd(A);   % Don't seem to be able to use the economy
                            % decomposition under Octave here
    else
	[U,D,V] = svd(A,0); % Under MATLAB use the economy decomposition
    end
    
    info = D(2,2)/D(1,1);
    
 %--------------------------------------------------------------------------
% Function to check argument values and set defaults   

    function [x1, x2, npts] = checkargs(arg);
    
    if length(arg) == 2
        x1 = arg{1};
        x2 = arg{2};
        if ~all(size(x1)==size(x2))
            error('x1 and x2 must have the same size');
        elseif size(x1,1) ~= 3
            error('x1 and x2 must be 3xN');
        end
        
    elseif length(arg) == 1
        if size(arg{1},1) ~= 6
            error('Single argument x must be 6xN');
        else
            x1 = arg{1}(1:3,:);
            x2 = arg{1}(4:6,:);
        end
    else
        error('Wrong number of arguments supplied');
    end
      
    npts = size(x1,2);