#include "StdAfx.h"
#include <ctime>
#include <cstdlib>


#define _MATLAB_  

struct list
	{
		int data;
		struct list *next;
};
double * readMat(int *numOfRows, int *numOfCols, char const *fileName);
void listDestruct(list * listToDestruct);
void printToFile(double const * clustering, list * nonDeletedIndeces, char const *fileName,int nMatches);
void extractingClustering(double * extractedClustering,double const * clustering,list * nonDeletedIndeces,int nMatches,int const *dim);
void extractingClusteringDimensions(double const * clustering,list * nonDeletedIndeces,int nMatches,int * dim);
void AgglomerativeClustering (double * clustering,list * nonDeletedIndeces,list ** nonDeletedArray,double const * des,double const * locInd,double clusteringThreshold, int clusteringMinimumSize,int Nmatches);

#ifndef _MATLAB_
int main(int argc, char* argv[])
{
	/*
		possible arguments:
		1. Descriptors file name
		2. Clustering threshold file name		
		3. ClusteringMinimumSizeName file name		
		4. locInd file name		
		5. Clustering file name		
	*/
	char const * argDesName = "descriptors.txt";
	char const * argClusteringThresholdName = "clusteringThreshold.txt";
	char const * argClusteringMinimumSizeName = "clusteringMinimumSize.txt";
	char const * argLocIndName = "locInd.txt";
	char const * argClusteringName = "clustering.txt";
	int nMatches,temp;

	if (argc >= 6){
		argClusteringName  = argv[5];
	}
	if (argc >= 5){
		argClusteringMinimumSizeName  = argv[4];
	}
	if (argc >= 4){
		argLocIndName  = argv[3];
	}
	if (argc >= 3){
		argClusteringThresholdName  = argv[2];
	}
	if (argc >= 2){
		argDesName  = argv[1];
	}	
	double * des = readMat(&temp,&nMatches,argDesName); //load descriptors;	
	double * clusteringThreshold = readMat(&temp, &temp,argClusteringThresholdName); //load ClusteringThreshold;
	double * clusteringMinimumSize = readMat(&temp, &temp,argClusteringMinimumSizeName); //load ClusteringMinimumSize;
	double * locInd = readMat(&temp, &temp,argLocIndName); //load LocInd;
	double * clustering = new double[nMatches*nMatches];
	list * nonDeletedIndeces = new list;
	list ** nonDeletedArray = new list*[nMatches];	

	for (int i=0 ; i<nMatches ; i++)
	{
		for (int j=0 ; j<nMatches ; j++)
		{		
			clustering[j*nMatches+i]=0;
		}
	}
	for (int j=0; j<nMatches ; j++)
	{
		clustering[j*nMatches]= j+1;
	}	
	AgglomerativeClustering(clustering,nonDeletedIndeces,nonDeletedArray,des,locInd,clusteringThreshold[0],(int) clusteringMinimumSize[0],nMatches);
	printToFile(clustering,nonDeletedIndeces,argClusteringName,nMatches);
	delete [] clustering;
	list * tempList = nonDeletedIndeces;
	while ((*tempList).data != -1)
	{
		listDestruct(nonDeletedArray[(*tempList).data]);
		tempList = (*tempList).next;
	}
	delete [] nonDeletedArray;
	listDestruct(nonDeletedIndeces);
	return 0;
}
#else
#include <matrix.h>
#include <mex.h>   

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{   
	double *des, *clusteringThreshold, *clusteringMinimumSize, *locInd, *extractedClustering;
	const mwSize *dims;
    int nMatches, dim[2];

	if(nrhs!=4) 
		mexErrMsgTxt("Four inputs required.");
	if(nlhs!=1) 
		mexErrMsgTxt("One output required.");

	//figure out dimensions
	dims = mxGetDimensions(prhs[0]);
    nMatches = (int)dims[1];

	////associate pointers
	des = mxGetPr(prhs[0]);
	clusteringThreshold = mxGetPr(prhs[1]);
	clusteringMinimumSize = mxGetPr(prhs[2]);
	locInd = mxGetPr(prhs[3]);
		
	double * clustering = new double[nMatches*nMatches];
	list * nonDeletedIndeces = new list;
	list ** nonDeletedArray = new list*[nMatches];
	for (int i=0 ; i<nMatches ; i++)
	{
		for (int j=0 ; j<nMatches ; j++)
		{		
			clustering[j*nMatches+i]=0;
		}
	}
	for (int j=0; j<nMatches ; j++)
	{
		clustering[j*nMatches]= j+1;
	}
	AgglomerativeClustering(clustering,nonDeletedIndeces,nonDeletedArray,des,locInd,clusteringThreshold[0],(int) clusteringMinimumSize[0],nMatches);	
	extractingClusteringDimensions(clustering,nonDeletedIndeces,nMatches,dim);
	
	//associate outputs
	plhs[0] = mxCreateDoubleMatrix(dim[0],dim[1],mxREAL);
	extractedClustering = mxGetPr(plhs[0]);  	
	extractingClustering(extractedClustering,clustering,nonDeletedIndeces,nMatches,dim);
	delete [] clustering;
	list * temp = nonDeletedIndeces;
	while ((*temp).data != -1)
	{
		listDestruct(nonDeletedArray[(*temp).data]);
		temp = (*temp).next;
	}
	delete [] nonDeletedArray;
	listDestruct(nonDeletedIndeces);
	return;
}
#endif

void whichClustersToMerge(double const * distanceMatrix,list * nonDeletedIndeces,list ** nonDeletedMatrix, int * mergingIndeces, double clusteringThreshold, int nMatches)
{
	double maxValue = -HUGE_VAL;
	list * ind1_list = nonDeletedIndeces;
	list * ind2_list;
	
	while ((*ind1_list).data != -1)
	{
		ind2_list = nonDeletedMatrix[(*ind1_list).data];
		while ((*ind2_list).data != -1)
		{
			if (distanceMatrix[(*ind1_list).data*nMatches +(*ind2_list).data]>maxValue)
			{
				maxValue = distanceMatrix[(*ind1_list).data*nMatches +(*ind2_list).data];
				mergingIndeces[0] = (*ind1_list).data;
				mergingIndeces[1] = (*ind2_list).data;
			}			
			ind2_list = (*ind2_list).next;
		}
		ind1_list = (*ind1_list).next;
	}
	if (maxValue <= clusteringThreshold)
	{
		mergingIndeces[0] = nMatches;
		mergingIndeces[1] = nMatches;
	}
}

void AgglomerativeClusteringInitialization (double * distanceMatrix,list * nonDeletedIndeces,list ** nonDeletedMat,double const * des, int nMatches, double clusteringThreshold)
{		
	list * temp = nonDeletedIndeces;	
	for (int i=0 ; i<nMatches ; i++)
	{
		(*temp).data = i;
		(*temp).next = new list;
		temp = (*temp).next;
	}
	(*temp).data = -1;
	for (int i=0 ; i<nMatches ; i++)
	{
		for (int j=0 ; j<i ; j++)
		{		
			double multipication = 0;
			for (int des_ind=0; des_ind<128 ; des_ind++)
			{
				multipication += des[j*128+des_ind] * des[i*128+des_ind];
			}
			distanceMatrix[i*nMatches+j]=multipication;
			distanceMatrix[j*nMatches+i]=multipication;			
		}		
	}
	for (int i=0 ; i<nMatches ; i++)
	{
		nonDeletedMat[i] = new list;
		temp = nonDeletedMat[i];
		for (int j=0 ; j<nMatches ; j++)
		{		
			if ((distanceMatrix[i*nMatches+j] > clusteringThreshold) && (i != j))
			{
				(*temp).data = j;
				(*temp).next = new list;
				temp = (*temp).next;
				
			}
		}
		(*temp).data = -1;
	}
}

void listDestruct(list * listToDestruct)
{
	while ((*listToDestruct).data != -1)
	{
		list * deleted_ind = listToDestruct;
		listToDestruct = (*listToDestruct).next;
		delete deleted_ind;
	}
	delete listToDestruct;
}

void clusterMerging(double * clustering,double * distanceMatrix,list * nonDeletedIndeces, list ** nonDeletedArray, int * mergingIndeces,int nMatches)
{
	int counter = 0, i = 0;
	list * new_list  = new list;
	list * temp = new_list, * temp2;
	list * ind_list1 = nonDeletedArray[mergingIndeces[0]];
	list * ind_list2 = nonDeletedArray[mergingIndeces[1]];
	while (clustering[mergingIndeces[0]*nMatches+counter] > 0)
	{
		counter++;
	}
	while (clustering[mergingIndeces[1]*nMatches+i] > 0)
	{
		clustering[mergingIndeces[0]*nMatches+counter] = clustering[mergingIndeces[1]*nMatches+i];
		i++;
		counter++;
	}	
	while (((*ind_list1).data != -1) || ((*ind_list2).data != -1))
	{
		if ((*ind_list1).data == (*ind_list2).data)
		{
			if (distanceMatrix[mergingIndeces[1]*nMatches+(*ind_list1).data] < distanceMatrix[mergingIndeces[0]*nMatches+(*ind_list1).data])			
			{
				distanceMatrix[mergingIndeces[0]*nMatches+(*ind_list1).data] = distanceMatrix[mergingIndeces[1]*nMatches+(*ind_list1).data];
				distanceMatrix[(*ind_list1).data*nMatches+mergingIndeces[0]] = distanceMatrix[mergingIndeces[1]*nMatches+(*ind_list1).data];
			}
			temp2 = nonDeletedArray[(*ind_list1).data];
			while (((*temp2).data != mergingIndeces[1]))// && ((*temp2).data != -1))
				temp2 = (*temp2).next;
			(*temp).data = (*ind_list1).data;
			(*temp).next = new list;
			temp = (*temp).next;
			ind_list1 = (*ind_list1).next;
			ind_list2 = (*ind_list2).next;			
		} else if ((((*ind_list1).data < (*ind_list2).data) && (*ind_list1).data != -1) || (*ind_list2).data == -1)
		{
			temp2 = nonDeletedArray[(*ind_list1).data];				
			while (((*temp2).data != mergingIndeces[0]) && ((*temp2).data != mergingIndeces[1]))// && ((*temp2).data != -1))
				temp2 = (*temp2).next;			
			ind_list1 = (*ind_list1).next;
		} else {
			temp2 = nonDeletedArray[(*ind_list2).data];				
			while (((*temp2).data != mergingIndeces[0]) && ((*temp2).data != mergingIndeces[1]))// && ((*temp2).data != -1))
				temp2 = (*temp2).next;
			ind_list2 = (*ind_list2).next;
		}					
		list * deleted_ind = (*temp2).next;
		(*temp2).data = (*(*temp2).next).data;
		(*temp2).next = (*(*temp2).next).next;
		delete deleted_ind;
	}
	(*temp).data = -1;

	temp = nonDeletedIndeces;
	while ((*(*temp).next).data != mergingIndeces[1])
		temp = (*temp).next;
	list * deleted_ind = (*temp).next;
	(*temp).next = (*(*temp).next).next;
	delete deleted_ind;

	listDestruct(nonDeletedArray[mergingIndeces[0]]);
	listDestruct(nonDeletedArray[mergingIndeces[1]]);
	nonDeletedArray[mergingIndeces[0]] = new_list;
}

void extractingClusteringDimensions(double const * clustering,list * nonDeletedIndeces,int nMatches,int * dim)
{
	list * ind_list = nonDeletedIndeces;	
	dim[0] = 0;
	dim[1] = 0;
	while ((*ind_list).data != -1)
	{
		int i = 0;
		while (clustering[(*ind_list).data*nMatches+i] > 0)
		{
			i++;
		}
		if (i > dim[0])
		{
			dim[0] = i;
		}
		dim[1]++;
		ind_list = (*ind_list).next;
	}
}

void extractingClustering(double * extractedClustering,double const * clustering,list * nonDeletedIndeces,int nMatches,int const *dim)
{
	list * ind_list = nonDeletedIndeces;	
	for (int j=0 ; j<dim[1] ; j++)
	{
		for (int i=0 ; i<dim[0] ; i++)
		{
			extractedClustering[j*dim[0]+i] = clustering[(*ind_list).data*nMatches+i];
		}
		ind_list = (*ind_list).next;
	}
}

void printToFile(double const * clustering, list * nonDeletedIndeces, char const *fileName,int nMatches)
{	
	int dim[2];
	extractingClusteringDimensions(clustering,nonDeletedIndeces,nMatches,dim);
	ofstream file(fileName);
	file << dim[0] << endl;
	file << dim[1] << endl;
	list * ind_list = nonDeletedIndeces;	
	for (int j=0 ; j<dim[1] ; j++)
	{
		for (int i=0 ; i<dim[0] ; i++)
		{
			file << clustering[(*ind_list).data*nMatches+i] << endl;
		}
		ind_list = (*ind_list).next;
	}
}

double * readMat(int *numOfRows, int *numOfCols, char const *fileName)
{
	ifstream file(fileName);
	if (! file){
		cerr << "Could not open file: " << fileName;
		exit(1);
	}	
	file >> *numOfRows;
	file >> *numOfCols;
	double *mat;
	mat = new double[(*numOfRows)*(*numOfCols)];
	for (int j=0 ; j<(*numOfCols) ; j++)
	{
		for (int i=0 ; i<(*numOfRows) ; i++)
		{
			file >> mat[j*(*numOfRows)+i];;
		}
	}
	return mat;
}

void AgglomerativeClustering (double * clustering,list * nonDeletedIndeces,list ** nonDeletedMatrix,double const * des,double const * locInd,double clusteringThreshold, int clusteringMinimumSize,int nMatches)
{	
	int mergingIndeces[2];	
	double * distanceMatrix = new double[nMatches*nMatches];
	AgglomerativeClusteringInitialization(distanceMatrix,nonDeletedIndeces,nonDeletedMatrix,des,nMatches,clusteringThreshold);		
	whichClustersToMerge(distanceMatrix,nonDeletedIndeces,nonDeletedMatrix,mergingIndeces,clusteringThreshold,nMatches);
	while (mergingIndeces[0] != nMatches)
	{
		clusterMerging(clustering,distanceMatrix,nonDeletedIndeces,nonDeletedMatrix,mergingIndeces,nMatches);
		whichClustersToMerge(distanceMatrix,nonDeletedIndeces,nonDeletedMatrix,mergingIndeces,clusteringThreshold,nMatches);
	}
	delete [] distanceMatrix;
}