#include "StdAfx.h"
#include <ctime>
#include <cstdlib>

#define SQR(a)   ((a)*(a))
#define NR_END 1
#define FREE_ARG char*
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
static double dsqrarg;
#define DSQR(a) ((dsqrarg=(a)) == 0.0 ? 0.0 : dsqrarg*dsqrarg)
static double dmaxarg1,dmaxarg2;
#define DMAX(a,b) (dmaxarg1=(a),dmaxarg2=(b),(dmaxarg1) > (dmaxarg2) ?\
        (dmaxarg1) : (dmaxarg2))
static int iminarg1,iminarg2;
#define IMIN(a,b) (iminarg1=(a),iminarg2=(b),(iminarg1) < (iminarg2) ?\
        (iminarg1) : (iminarg2))

#define _MATLAB_

void removingPairsGivingSame_F(bool * nonRedundant,int * newN,double const * pairs,int N);
void FFrom_points(double * Fs,int * FDims,bool const * nonRedundant,int N,double const * pairs, int GenerateLength);
double * readMat(int *numOfRows, int *numOfCols, char const *fileName);
void printMatToFile(double const * mat, char const *fileName, int const * dims);
void dsvdcmp(double *a, int m, int n, double w[], double *v);

#ifndef _MATLAB_

int main(int argc, char* argv[])
{
	/*
		possible arguments:
		1. pairs file name
		2. GenerateLength file name
		3. Fs file name
	*/
	char const * argPairs = "pairs.txt";
	char const * argGenerateLength = "GenerateLength.txt";
	char const * argFs = "Fs.txt";		

	if (argc >= 4){
		argFs  = argv[3];
	}
	if (argc >= 3){
		argGenerateLength  = argv[2];
	}
	if (argc >= 2){
		argPairs  = argv[1];
	}
	int newN,N,temp;
	double *pairs = readMat(&temp,&N,argPairs);
	double *GenerateLength = readMat(&temp,&temp,argGenerateLength);
	bool * nonRedundant = new bool[N];
	removingPairsGivingSame_F(nonRedundant,&newN,pairs,N);

	int FDims[2];
	FDims[0] = 9;
	FDims[1] = newN*(newN-1)/2;
	double *Fs = new double[FDims[0]*FDims[1]];
	FFrom_points(Fs,FDims,nonRedundant,N,pairs,(int)GenerateLength[0]);
	printMatToFile(Fs,argFs,FDims);

	delete [] pairs;
	delete [] nonRedundant;
	delete [] Fs;
	return 0;
}
#else
#include <matrix.h>
#include <mex.h>   

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{   	
	double *pairs, *GenerateLength, *Fs;	
	const int *pairsDims;
	int FDims[2];
	
	/* check proper input and output */
	if(nrhs!=2) 
		mexErrMsgTxt("2 inputs required.");
	if(nlhs!=1) 
		mexErrMsgTxt("One output required.");    
    
	//figure out dimensions	
	/*pairsDims = new int[2];*/
	pairsDims = mxGetDimensions(prhs[0]);
	
	////associate pointers
	pairs = mxGetPr(prhs[0]);
	GenerateLength = mxGetPr(prhs[1]);
	
	int newN;
	bool * nonRedundant = new bool[pairsDims[1]];
	removingPairsGivingSame_F(nonRedundant,&newN,pairs,pairsDims[1]);
	FDims[0] = 9;
	FDims[1] = newN*(newN-1)/2;

	//associate outputs
	plhs[0] = mxCreateDoubleMatrix(FDims[0],FDims[1],mxREAL);	
	Fs = mxGetPr(plhs[0]);

	FFrom_points(Fs,FDims,nonRedundant,pairsDims[1],pairs,(int)GenerateLength[0]);	
	/*delete [] pairsDims;*/
	return;
}
#endif

void findLocToInd(int * locToInd, int * N, double const * pairs, int Npairs, int offset)
{
	for (int i=0 ; i<2*Npairs; i++)
		locToInd[i] = Npairs;
	for (int i=0 ; i<Npairs; i++)
	{
		for (int offsetI=0 ; offsetI<2; offsetI++)
		{
			if (locToInd[offsetI+i*2] == Npairs)
			{			
				for (int j=i ; j<Npairs; j++)
					for (int offsetJ=0 ; offsetJ<2; offsetJ++)
						if ((pairs[offset+offsetI*12+i*28] == pairs[offset+offsetJ*12+j*28]) && (pairs[1+offset+offsetI*12+i*28] == pairs[1+offset+offsetJ*12+j*28]))
							locToInd[offsetJ+j*2] = *N;
				(*N)++;
			}
		}
	}
}

void removingPairsGivingSame_F(bool * nonRedundant,int * newN,double const * pairs,int N)
{
	int * locToInd1 = new int[2*N];
	int * locToInd2 = new int[2*N];
	int N1 = 0, N2 = 0, ind1, ind2, ind3, ind4, indLoc1, indLoc2, ind;
	findLocToInd(locToInd1,&N1,pairs,N,0);
	findLocToInd(locToInd2,&N2,pairs,N,6);
	bool * hash = new bool[(N1+1)*N1/2*(N2+1)*N2/2];
	(*newN) = N;
	for (int i=0 ; i<N; i++)
	{
		ind1 = max(locToInd1[i*2],locToInd1[1+i*2]);
		ind2 = min(locToInd1[i*2],locToInd1[1+i*2]);
		ind3 = max(locToInd2[i*2],locToInd2[1+i*2]);
		ind4 = min(locToInd2[i*2],locToInd2[1+i*2]);
		indLoc1 = (ind1+1)*ind1/2 + ind2;
		indLoc2 = (ind3+1)*ind3/2 + ind4;
		ind  = indLoc1 + indLoc2*(N1+1)*N1/2;
		if (hash[ind] == false)
		{
			nonRedundant[i] = true;
			hash[ind] = true;
		} else {
			nonRedundant[i] = false;			
			(*newN)--;
		}
	}
	delete [] locToInd1;
	delete [] locToInd2;
	delete [] hash;
}

void normalization(double * loc,double * T,double const * inputLoc,int offset,int ind1,int ind2,int N)
{
	double ori[2];
	ori[0] = 0; 
	ori[1] = 0;
	for (int i=0 ; i<8 ; i++)
	{
		ori[0] += inputLoc[offset+i*4+ind1*32];
		ori[0] += inputLoc[offset+i*4+ind2*32];
		ori[1] += inputLoc[1+offset+i*4+ind1*32];
		ori[1] += inputLoc[1+offset+i*4+ind2*32];
	}
	ori[0] /= N;
	ori[1] /= N;

	double sum=0;
	for (int  i=0 ; i<8 ; i++)
	{
		sum += sqrt(SQR((inputLoc[offset+i*4+ind1*32]-ori[0]))+SQR((inputLoc[1+offset+i*4+ind1*32]-ori[1])));
		sum += sqrt(SQR((inputLoc[offset+i*4+ind2*32]-ori[0]))+SQR((inputLoc[1+offset+i*4+ind2*32]-ori[1])));
	}
	double par=sqrt(2.0)/(sum/N);
	T[0] = par;
	T[1] = 0;
	T[2] = (-1)*ori[0]*par;
	T[3] = 0;
	T[4] = par;
	T[5] = (-1)*ori[1]*par;
	T[6] = 0;
	T[7] = 0;
	T[8] = 1;
	for (int i=0 ; i<8 ; i++)
	{
		loc[i*2]   = T[0]*inputLoc[offset+i*4+ind1*32] + T[1]*inputLoc[1+offset+i*4+ind1*32] + T[2];
		loc[16+i*2] = T[0]*inputLoc[offset+i*4+ind2*32] + T[1]*inputLoc[1+offset+i*4+ind2*32] + T[2];
		loc[1+i*2] = T[3]*inputLoc[offset+i*4+ind1*32] + T[4]*inputLoc[1+offset+i*4+ind1*32] + T[5];
		loc[17+i*2] = T[3]*inputLoc[offset+i*4+ind2*32] + T[4]*inputLoc[1+offset+i*4+ind2*32] + T[5];
	}
}

void NPointsAlg(double * tempF,double const * loc1,double const * loc2, int N)
{
	double * A = new double[10*(N+1)];
	double w[10]; 
	double vHelper[100];
	for (int i=0; i<100 ; i++)
		vHelper[i] = 0;
	for (int i=1; i<(N+1) ; i++)
	{
		A[i*10] = 0;
		A[1+i*10] = loc1[(i-1)*2]   * loc2[(i-1)*2];
		A[2+i*10] = loc1[1+(i-1)*2] * loc2[(i-1)*2];
		A[3+i*10] = loc2[(i-1)*2];
		A[4+i*10] = loc1[(i-1)*2]   * loc2[1+(i-1)*2];
		A[5+i*10] = loc1[1+(i-1)*2] * loc2[1+(i-1)*2];
		A[6+i*10] = loc2[1+(i-1)*2];
		A[7+i*10] = loc1[(i-1)*2];
		A[8+i*10] = loc1[1+(i-1)*2];
		A[9+i*10] = 1;
	}
	for (int i=0 ; i<10 ; i++)
	{
		A[i] = 0;
	}
	dsvdcmp(A,N,9,w,vHelper);
	double minVal = w[1];
	int minInd = 1;
	for (int i=2 ; i<10 ; i++)
	{
		if (minVal > w[i])
		{
			minVal = w[i];
			minInd = i;
		}		
	}
	for (int i=0 ; i<9 ; i++)
	{
		tempF[i] = vHelper[10+minInd+i*10];
	}
	delete [] A;
}

void renormalization(double * Fs,int count,double const * tempF,double const * T1,double const * T2)
{
	double FuncTa[9];
	FuncTa[0] = tempF[0]*T1[0] + tempF[1]*T1[3] + tempF[2]*T1[6];
	FuncTa[1] = tempF[0]*T1[1] + tempF[1]*T1[4] + tempF[2]*T1[7];
	FuncTa[2] = tempF[0]*T1[2] + tempF[1]*T1[5] + tempF[2]*T1[8];
	FuncTa[3] = tempF[3]*T1[0] + tempF[4]*T1[3] + tempF[5]*T1[6];
	FuncTa[4] = tempF[3]*T1[1] + tempF[4]*T1[4] + tempF[5]*T1[7];
	FuncTa[5] = tempF[3]*T1[2] + tempF[4]*T1[5] + tempF[5]*T1[8];
	FuncTa[6] = tempF[6]*T1[0] + tempF[7]*T1[3] + tempF[8]*T1[6];
	FuncTa[7] = tempF[6]*T1[1] + tempF[7]*T1[4] + tempF[8]*T1[7];
	FuncTa[8] = tempF[6]*T1[2] + tempF[7]*T1[5] + tempF[8]*T1[8];

	Fs[0 + count*9] = T2[0]*FuncTa[0] + T2[3]*FuncTa[3] + T2[6]*FuncTa[6];
	Fs[3 + count*9] = T2[0]*FuncTa[1] + T2[3]*FuncTa[4] + T2[6]*FuncTa[7];
	Fs[6 + count*9] = T2[0]*FuncTa[2] + T2[3]*FuncTa[5] + T2[6]*FuncTa[8];
	Fs[1 + count*9] = T2[1]*FuncTa[0] + T2[4]*FuncTa[3] + T2[7]*FuncTa[6];
	Fs[4 + count*9] = T2[1]*FuncTa[1] + T2[4]*FuncTa[4] + T2[7]*FuncTa[7];
	Fs[7 + count*9] = T2[1]*FuncTa[2] + T2[4]*FuncTa[5] + T2[7]*FuncTa[8];
	Fs[2 + count*9] = T2[2]*FuncTa[0] + T2[5]*FuncTa[3] + T2[8]*FuncTa[6];
	Fs[5 + count*9] = T2[2]*FuncTa[1] + T2[5]*FuncTa[4] + T2[8]*FuncTa[7];
	Fs[8 + count*9] = T2[2]*FuncTa[2] + T2[5]*FuncTa[5] + T2[8]*FuncTa[8];
}

void normalized_N_points_alg(double * Fs,int count, double const * inputLoc, int ind1, int ind2,int N)
{	
	double T1[9];
	double T2[9];
	double tempF[9];
	double * loc1 = new double[2*N];
	double * loc2 = new double[2*N];
	normalization(loc1,T1,inputLoc,0,ind1,ind2,N);
	normalization(loc2,T2,inputLoc,2,ind1,ind2,N);
	NPointsAlg(tempF,loc1,loc2,N);
	renormalization(Fs,count,tempF,T1,T2);
	delete [] loc1;
	delete [] loc2;
}

void additionalPointsGeneration(double * inputLoc,double const * pairs,int im,int nn,int i,int GenerateLength)
{
	double ang[3];
	ang[0] = 0;
	ang[1] = 2*M_PI/3;
	ang[2] = 2*M_PI*2/3;
	for (int j=0 ; j<3; j++)
	{
		inputLoc[  2*im+(j+1)*4+16*nn+i*32] = GenerateLength*cos(pairs[3+6*im+12*nn+i*28]+ang[j])*pairs[2+6*im+12*nn+i*28]+pairs[6*im+12*nn+i*28];        
		inputLoc[1+2*im+(j+1)*4+16*nn+i*32] = GenerateLength*sin(pairs[3+6*im+12*nn+i*28]+ang[j])*pairs[2+6*im+12*nn+i*28]+pairs[1+6*im+12*nn+i*28];
	}
}

void FFrom_points(double * Fs,int * FDims,bool const * nonRedundant,int N,double const * pairs, int GenerateLength)
{
	int count = 0, NinputLoc = 16;
	double * inputLoc = new double[32*N];
	for (int i=0 ; i<N; i++)
	{
		if (nonRedundant[i])
		{
			for (int im=0 ; im<2; im++)
			{
				for (int nn=0 ; nn<2; nn++)
				{
					inputLoc[  2*im+16*nn+i*32] = pairs[6*im+12*nn+i*28];
					inputLoc[1+2*im+16*nn+i*32] = pairs[1+6*im+12*nn+i*28];
					additionalPointsGeneration(inputLoc,pairs,im,nn,i,GenerateLength);
				}
			}
		}
	}
	for (int i=0 ; i<N; i++)
	{
		if (nonRedundant[i])
		{
			for (int j=0 ; j<i; j++)
			{
				if (nonRedundant[j])
				{
					normalized_N_points_alg(Fs,count,inputLoc,i,j,NinputLoc);
					count++;
				}
			}
		}
	}	
	delete [] inputLoc;
}

double * readMat(int *numOfRows, int *numOfCols, char const *fileName)
{
	ifstream file(fileName);
	if (! file){
		cerr << "Could not open file: " << fileName;
		exit(1);
	}	
	file >> *numOfRows;
	file >> *numOfCols;
	double *mat;
	mat = new double[(*numOfRows)*(*numOfCols)];
	for (int j=0 ; j<(*numOfCols) ; j++)
	{
		for (int i=0 ; i<(*numOfRows) ; i++)
		{
			file >> mat[j*(*numOfRows)+i];;
		}
	}
	return mat;
}

void printMatToFile(double const * mat, char const *fileName, int const * dims)
{
	ofstream file(fileName);
	file << dims[0] << endl;
	file << dims[1] << endl;
	for (int j=0 ; j<dims[1]; j++)
	{
		for (int i=0 ; i<dims[0]; i++)
		{
			file << mat[i + dims[0]*j] << endl;
		}
	}
}

double * dvector(long nl, long nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
	double *v;

	v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
	return v-nl+NR_END;
}

void free_dvector(double *v, long nl, long nh)
/* free a double vector allocated with dvector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

double dpythag(double a, double b)
{
	double absa,absb;
	absa=fabs(a);
	absb=fabs(b);
	if (absa > absb) return absa*sqrt(1.0+DSQR(absb/absa));
	else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+DSQR(absa/absb)));
	return 0.0;
}

void dsvdcmp(double *a, int m, int n, double w[], double *v)
{
	int flag,i,its,j,jj,k,l,nm;
	double anorm,c,f,g,h,s,scale,x,y,z,*rv1;

	rv1=dvector(1,n);
	g=scale=anorm=0.0;
	for (i=1;i<=n;i++) {
		l=i+1;
		rv1[i]=scale*g;
		g=s=scale=0.0;
		if (i <= m) {
			for (k=i;k<=m;k++) scale += fabs(a[i+k*10]);
			if (scale) {
				for (k=i;k<=m;k++) {
					a[i+k*10] /= scale;
					s += a[i+k*10]*a[i+k*10];
				}
				f=a[i+i*10];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i+i*10]=f-g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=i;k<=m;k++) s += a[i+k*10]*a[j+k*10];
					f=s/h;
					for (k=i;k<=m;k++) a[j+k*10] += f*a[i+k*10];
				}
				for (k=i;k<=m;k++) a[i+k*10] *= scale;
			}
		}
		w[i]=scale *g;
		g=s=scale=0.0;
		if (i <= m && i != n) {
			for (k=l;k<=n;k++) scale += fabs(a[k+i*10]);
			if (scale) {
				for (k=l;k<=n;k++) {
					a[k+i*10] /= scale;
					s += a[k+i*10]*a[k+i*10];
				}
				f=a[l+i*10];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[l+i*10]=f-g;
				for (k=l;k<=n;k++) rv1[k]=a[k+i*10]/h;
				for (j=l;j<=m;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[k+j*10]*a[k+i*10];
					for (k=l;k<=n;k++) a[k+j*10] += s*rv1[k];
				}
				for (k=l;k<=n;k++) a[k+i*10] *= scale;
			}
		}
		anorm=DMAX(anorm,(fabs(w[i])+fabs(rv1[i])));
	}
	for (i=n;i>=1;i--) {
		if (i < n) {
			if (g) {
				for (j=l;j<=n;j++) v[i+j*10]=(a[j+i*10]/a[l+i*10])/g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[k+i*10]*v[j+k*10];
					for (k=l;k<=n;k++) v[j+k*10] += s*v[i+k*10];
				}
			}
			for (j=l;j<=n;j++) v[j+i*10]=v[i+j*10]=0.0;
		}
		v[i+i*10]=1.0;
		g=rv1[i];
		l=i;
	}
	for (i=IMIN(m,n);i>=1;i--) {
		l=i+1;
		g=w[i];
		for (j=l;j<=n;j++) a[j+i*10]=0.0;
		if (g) {
			g=1.0/g;
			for (j=l;j<=n;j++) {
				for (s=0.0,k=l;k<=m;k++) s += a[i+k*10]*a[j+k*10];
				f=(s/a[i+i*10])*g;
				for (k=i;k<=m;k++) a[j+k*10] += f*a[i+k*10];
			}
			for (j=i;j<=m;j++) a[i+j*10] *= g;
		} else for (j=i;j<=m;j++) a[i+j*10]=0.0;
		++a[i+i*10];
	}
	for (k=n;k>=1;k--) {
		for (its=1;its<=30;its++) {
			flag=1;
			for (l=k;l>=1;l--) {
				nm=l-1;
				if ((double)(fabs(rv1[l])+anorm) == anorm) {
					flag=0;
					break;
				}
				if ((double)(fabs(w[nm])+anorm) == anorm) break;
			}
			if (flag) {
				c=0.0;
				s=1.0;
				for (i=l;i<=k;i++) {
					f=s*rv1[i];
					rv1[i]=c*rv1[i];
					if ((double)(fabs(f)+anorm) == anorm) break;
					g=w[i];
					h=dpythag(f,g);
					w[i]=h;
					h=1.0/h;
					c=g*h;
					s = -f*h;
					for (j=1;j<=m;j++) {
						y=a[nm+j*10];
						z=a[i+j*10];
						a[nm+j*10]=y*c+z*s;
						a[i+j*10]=z*c-y*s;
					}
				}
			}
			z=w[k];
			if (l == k) {
				if (z < 0.0) {
					w[k] = -z;
					for (j=1;j<=n;j++) v[k+j*10] = -v[k+j*10];
				}
				break;
			}
			x=w[l];
			nm=k-1;
			y=w[nm];
			g=rv1[nm];
			h=rv1[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g=dpythag(f,1.0);
			f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c=s=1.0;
			for (j=l;j<=nm;j++) {
				i=j+1;
				g=rv1[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=dpythag(f,h);
				rv1[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g = g*c-x*s;
				h=y*s;
				y *= c;
				for (jj=1;jj<=n;jj++) {
					x=v[j+jj*10];
					z=v[i+jj*10];
					v[j+jj*10]=x*c+z*s;
					v[i+jj*10]=z*c-x*s;
				}
				z=dpythag(f,h);
				w[j]=z;
				if (z) {
					z=1.0/z;
					c=f*z;
					s=h*z;
				}
				f=c*g+s*y;
				x=c*y-s*g;
				for (jj=1;jj<=m;jj++) {
					y=a[j+jj*10];
					z=a[i+jj*10];
					a[j+jj*10]=y*c+z*s;
					a[i+jj*10]=z*c-y*s;
				}
			}
			rv1[l]=0.0;
			rv1[k]=f;
			w[k]=x;
		}
	}
	free_dvector(rv1,1,n);
}
