#include "StdAfx.h"
#include <ctime>
#include <cstdlib>

#define SQR(a)   ((a)*(a))
struct list
	{
		int data;
		struct list *next;
};

#define _MATLAB_

void calculateGridTableDimensions(int * gridTableDimensions, double const * loc, int gridCellSize, int nRows, int nCols);
void findAllPossibleKeypointMatches(double * possibleKeypointMatches,double *possible2KeypointMatches,double * possible2KeypointMatchesDescriptors,double const *loc1,double const *loc2,double const *inModes1,double const *inModes2,double const *match,double const *des1,double const *des2,int gridCellSize,int KnnScalesMax,int KScales,double thresholdSamePoint,int knn,int nLoc1,int nLoc2,int nMatches,int const * inModes1Dims,int const * inModes2Dims,int *PossibleKeypointMatchesDims, int * possible2KeypointMatchesDims,int * possible2KeypointMatchesDescriptorsDims);
void findAllPossible2KeypointMatches(double * possible2KeypointMatches, double * possible2KeypointMatchesDescriptors, double const * possibleKeypointMatches,int const * gridTable1Loc,int const * NgridTable1Loc,int const * gridTable2Loc,int const * NgridTable2Loc,double const * loc1, double const * loc2, int nLoc1, int nLoc2,int const * loc1ToPKM,int const * loc2ToPKM,int const * Nloc1ToPKM,int const * Nloc2ToPKM,int gridCellSize, int KnnScalesMax, int KScales, double thresholdSamePoint,int knn, int * gridTableDimensions1,int * gridTableDimensions2, int *possibleKeypointMatchesDims, int *possible2KeypointMatchesDims, int *possible2KeypointMatchesDescriptorsDims,double const * match,int nMatches,double const *inModes1,double const *inModes2,int const * inModes1Dims,int const * inModes2Dims);
double * readMat(int *numOfRows, int *numOfCols, char const *fileName);
void printMatToFile(double const * mat, char const *fileName, int const * dims);
void print3DToFile(double const * mat,char const *fileName,int const * dims);

#ifndef _MATLAB_

int main(int argc, char* argv[])
{
	/*
		possible arguments:
		1. loc1 file name
		2. loc2 file name
		3. inModes1 file name
		4. inModes2 file name
		5. match file name
		6. des1 file name
		7. des2 file name
		8. gridCellSize file name
		9. argKnnScales file name
		10. argKScalesName file name
		11. argThresholdSamePoint file name
		12. argKNN file name
	*/
	char const * argLoc1Name = "loc1.txt";
	char const * argLoc2Name = "loc2.txt";
	char const * argInModes1Name = "inModes1.txt";
	char const * argInModes2Name = "inModes2.txt";
	char const * argMatchesName = "matches.txt";
	char const * argDes1Name = "des1.txt";
	char const * argDes2Name = "des2.txt";
	char const * argGridCellSizeName = "gridCellSize.txt";
	char const * argKnnScalesMaxName = "KnnScalesMax.txt";
	char const * argKScalesName = "KScales.txt";	
	char const * argThresholdSamePoint = "ThresholdSamePoint.txt";	
	char const * argKNN = "KNN.txt";		

	if (argc >= 11){
		argKScalesName  = argv[10];
	}
	if (argc >= 10){
		argKnnScalesMaxName  = argv[9];
	}
	if (argc >= 9){
		argGridCellSizeName  = argv[8];
	}
	if (argc >= 8){
		argDes2Name  = argv[7];
	}
	if (argc >= 7){
		argDes1Name  = argv[6];
	}
	if (argc >= 6){
		argMatchesName  = argv[5];
	}
	if (argc >= 5){
		argInModes2Name  = argv[4];
	}
	if (argc >= 4){
		argInModes1Name  = argv[3];
	}
	if (argc >= 3){
		argLoc2Name  = argv[2];
	}
	if (argc >= 2){
		argLoc1Name  = argv[1];
	}
	int nLoc1,nLoc2,nMatches,temp,maxLength;
	int inModes1Dims[2];
	int inModes2Dims[2];
	int possibleKeypointMatchesDims[2];
	int possible2KeypointMatchesDims[2];
	int possible2KeypointMatchesDescriptorsDims[2];
	double *loc1 = readMat(&temp,&nLoc1,argLoc1Name);
	double *loc2 = readMat(&temp,&nLoc2,argLoc2Name);
	double *inModes1 = readMat(inModes1Dims,inModes1Dims+1,argInModes1Name);
	double *inModes2 = readMat(inModes2Dims,inModes2Dims+1,argInModes2Name);
	double *match = readMat(&temp,&nMatches,argMatchesName);	
	double *des1 = readMat(&temp,&temp,argDes1Name);
	double *des2 = readMat(&temp,&temp,argDes2Name);
	double *gridCellSize = readMat(&temp,&temp,argGridCellSizeName);
	double *KnnScalesMax = readMat(&temp,&temp,argKnnScalesMaxName);
	double *KScales = readMat(&temp,&temp,argKScalesName);
	double *thresholdSamePoint = readMat(&temp,&temp,argThresholdSamePoint);
	double *knn = readMat(&temp,&temp,argKNN);
	
	maxLength = 2*max(nLoc1/inModes1Dims[0],nLoc2/inModes2Dims[0])*inModes2Dims[0]*inModes1Dims[0];
	double * possibleKeypointMatches = new double[maxLength*12];
	double * possible2KeypointMatches = new double[maxLength*28];
	double * possible2KeypointMatchesDescriptors = new double[maxLength*7];
	findAllPossibleKeypointMatches(possibleKeypointMatches,possible2KeypointMatches,possible2KeypointMatchesDescriptors,loc1,loc2,inModes1,inModes2,match,des1,des2,(int) gridCellSize[0],(int) KnnScalesMax[0],(int) KScales[0],thresholdSamePoint[0],(int) knn[0],nLoc1,nLoc2,nMatches,inModes1Dims,inModes2Dims,possibleKeypointMatchesDims,possible2KeypointMatchesDims,possible2KeypointMatchesDescriptorsDims);
	printMatToFile(possibleKeypointMatches,"possibleKeypointMatches.txt",possibleKeypointMatchesDims);
	printMatToFile(possible2KeypointMatches,"possible2KeypointMatches.txt",possible2KeypointMatchesDims);
	printMatToFile(possible2KeypointMatchesDescriptors,"possible2KeypointMatchesDescriptors.txt",possible2KeypointMatchesDescriptorsDims);

	delete [] loc1;
	delete [] loc2;
	delete [] inModes1;
	delete [] inModes2;
	delete [] match;
	delete [] des1;
	delete [] des2;
	delete [] gridCellSize;
	delete [] possibleKeypointMatches;
	delete [] possible2KeypointMatches;
	delete [] possible2KeypointMatchesDescriptors;
	return 0;
}
#else
#include <matrix.h>
#include <mex.h>   

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{   	
	double *loc1, *loc2, *inModes1, *inModes2, *match, *des1, *des2, *gridCellSize, *knnScalesMax, *kScales, *thresholdSamePoint,*knn,*possibleKeypointMatches,*possible2KeypointMatches,*possible2KeypointMatchesDescriptors,*newptr;	
	const int *loc1Dims, *loc2Dims, *matchesDims, *inModes1Dims, *inModes2Dims;
	int possibleKeypointMatchesDims[2];
	int possible2KeypointMatchesDims[2];
	int possible2KeypointMatchesDescriptorsDims[2];
	
	size_t nbytes;
	/* check proper input and output */
	if(nrhs!=12) 
		mexErrMsgTxt("12 inputs required.");
	if(nlhs!=3) 
		mexErrMsgTxt("Three output required.");    
    
	//figure out dimensions	
	/*loc1Dims = new int[2];
	loc2Dims = new int[2];
	matchesDims = new int[2];
	inModes1Dims = new int[2];
	inModes2Dims = new int[2];*/
	loc1Dims = mxGetDimensions(prhs[0]);
	loc2Dims = mxGetDimensions(prhs[1]);
	inModes1Dims = mxGetDimensions(prhs[2]);
	inModes2Dims = mxGetDimensions(prhs[3]);
	matchesDims = mxGetDimensions(prhs[4]);	
	
	////associate pointers
	loc1 = mxGetPr(prhs[0]);
	loc2 = mxGetPr(prhs[1]);
	inModes1 = mxGetPr(prhs[2]);
	inModes2 = mxGetPr(prhs[3]);
	match = mxGetPr(prhs[4]);
	des1 = mxGetPr(prhs[5]);
	des2 = mxGetPr(prhs[6]);
	gridCellSize = mxGetPr(prhs[7]);
	knnScalesMax = mxGetPr(prhs[8]);
	kScales = mxGetPr(prhs[9]);
	thresholdSamePoint = mxGetPr(prhs[10]);
	knn = mxGetPr(prhs[11]);
	
	int maxLength = 2*max(loc1Dims[1]/inModes1Dims[0],loc2Dims[1]/inModes2Dims[0])*inModes2Dims[0]*inModes1Dims[0];
	possibleKeypointMatches = (double *) mxMalloc(maxLength*12*sizeof(*possibleKeypointMatches));
	possible2KeypointMatches = (double *) mxMalloc(maxLength*28*sizeof(*possibleKeypointMatches));
	possible2KeypointMatchesDescriptors = (double *) mxMalloc(maxLength*7*sizeof(*possibleKeypointMatches));
	findAllPossibleKeypointMatches(possibleKeypointMatches,possible2KeypointMatches,possible2KeypointMatchesDescriptors,loc1,loc2,inModes1,inModes2,match,des1,des2,(int) gridCellSize[0],knnScalesMax[0],kScales[0],thresholdSamePoint[0],knn[0],loc1Dims[1],loc2Dims[1],matchesDims[1],inModes1Dims,inModes2Dims,possibleKeypointMatchesDims,possible2KeypointMatchesDims,possible2KeypointMatchesDescriptorsDims);
	
	//associate outputs
	plhs[0] = mxCreateDoubleMatrix(possibleKeypointMatchesDims[0],possibleKeypointMatchesDims[1],mxREAL);	
	plhs[1] = mxCreateDoubleMatrix(possible2KeypointMatchesDims[0],possible2KeypointMatchesDims[1],mxREAL);
	plhs[2] = mxCreateDoubleMatrix(possible2KeypointMatchesDescriptorsDims[0],possible2KeypointMatchesDescriptorsDims[1],mxREAL);

	mxFree(mxGetPr(plhs[0]));
	nbytes = possibleKeypointMatchesDims[0] * possibleKeypointMatchesDims[1] * sizeof(*possibleKeypointMatches);
	newptr = (double *) mxRealloc(possibleKeypointMatches, nbytes);
	mxSetPr(plhs[0],newptr);
	mxFree(mxGetPr(plhs[1]));
	nbytes = possible2KeypointMatchesDims[0] * possible2KeypointMatchesDims[1] * sizeof(*possible2KeypointMatches);
	newptr = (double *) mxRealloc(possible2KeypointMatches, nbytes);
	mxSetPr(plhs[1],newptr);
	mxFree(mxGetPr(plhs[2]));
	nbytes = possible2KeypointMatchesDescriptorsDims[0] * possible2KeypointMatchesDescriptorsDims[1] * sizeof(*possible2KeypointMatchesDescriptors);
	newptr = (double *) mxRealloc(possible2KeypointMatchesDescriptors, nbytes);
	mxSetPr(plhs[2],newptr);
	
	/*delete loc1Dims;
	delete loc2Dims;
	delete matchesDims;
	delete inModes1Dims;
	delete inModes2Dims;*/
	return;
}
#endif

double * readMat(int *numOfRows, int *numOfCols, char const *fileName)
{
	ifstream file(fileName);
	if (! file){
		cerr << "Could not open file: " << fileName;
		exit(1);
	}	
	file >> *numOfRows;
	file >> *numOfCols;
	double *mat;
	mat = new double[(*numOfRows)*(*numOfCols)];
	for (int j=0 ; j<(*numOfCols) ; j++)
	{
		for (int i=0 ; i<(*numOfRows) ; i++)
		{
			file >> mat[j*(*numOfRows)+i];;
		}
	}
	return mat;
}

void printMatToFile(double const * mat, char const *fileName, int const * dims)
{
	ofstream file(fileName);
	file << dims[0] << endl;
	file << dims[1] << endl;
	for (int j=0 ; j<dims[1]; j++)
	{
		for (int i=0 ; i<dims[0]; i++)
		{
			file << mat[i + dims[0]*j] << endl;
		}
	}
}

void print3DToFile(double const * mat,char const *fileName,int const * dims)
{
	ofstream file(fileName);
	file << dims[0] << endl;
	file << dims[1] << endl;	
	file << dims[2] << endl;
	for (int i=0 ; i<dims[0]; i++)
	{
		for (int j=0 ; j<dims[1]; j++)
		{
			for (int k=0 ; k<dims[2]; k++)
			{
				file << mat[i + j*dims[0]+ k*dims[0]*dims[1]] << endl;
			}
		}
	}
}

void calculateGridTableDimensions(int * gridTableDimensions, double const * loc, int gridCellSize, int nRows, int nCols)
{
	double maxVal;
	for (int i=0 ; i<2; i++)
	{
		maxVal = 0;
		for (int j=0 ; j<nCols; j++)
		{
			if (loc[i+nRows*j] > maxVal)
				maxVal = loc[i+nRows*j];
		}
		gridTableDimensions[i] = (int) (maxVal/gridCellSize + 0.9999999999);
	}
}

void getSamePointMatching(int *matching,list *ind1,list *ind2,double const *des1,double const *des2,int nLoc1,int nLoc2)
{
	double multipication,maxVal = -1;
	int des_ind;
	list *tempList2,*tempList1 = ind1;
	while ((*tempList1).data != -1)
	{
		tempList2 = ind2;
		while ((*tempList2).data != -1)
		{
			multipication = 0;
			for (des_ind=0; des_ind<128 ; des_ind++)
				multipication += des1[des_ind+(*tempList1).data*128] * des2[des_ind+(*tempList2).data*128];
			if (multipication > maxVal)
			{
				maxVal = multipication;
				matching[0] = (*tempList1).data;
				matching[1] = (*tempList2).data;
			}
			tempList2 = (*tempList2).next;
		}
		tempList1 = (*tempList1).next;
	}
}

void divideLocsIntoGrid(int * gridTableLoc,int * N, int nLoc,double const * loc,int const * dims,int gridCellSize)
{
	int ind;
	for (int j=0 ; j<dims[1]; j++)
		for (int i=0 ; i<dims[0]; i++)
			N[i + j*dims[0]] = 0;
	for (int i=0; i<nLoc; i++)
	{
		ind = (int)(loc[0+i*5]/gridCellSize - 0.0000000001) + ((int)(loc[1+i*5]/gridCellSize - 0.0000000001))* dims[0];
		gridTableLoc[ind + N[ind] * dims[0]* dims[1]] = i;
		(N[ind])++;
	}
}

void findAllPossibleKeypointMatches(double * possibleKeypointMatches,double *possible2KeypointMatches,double * possible2KeypointMatchesDescriptors,double const *loc1,double const *loc2,double const *inModes1,double const *inModes2,double const *match,double const *des1,double const *des2,int gridCellSize,int KnnScalesMax,int KScales,double thresholdSamePoint,int knn,int nLoc1,int nLoc2,int nMatches,int const * inModes1Dims,int const * inModes2Dims,int *PossibleKeypointMatchesDims, int * possible2KeypointMatchesDims,int * possible2KeypointMatchesDescriptorsDims)
{	
	int gridTableDimensions1[3];
	int gridTableDimensions2[3];
	calculateGridTableDimensions(gridTableDimensions1,loc1,(int) gridCellSize,5,nLoc1);
	calculateGridTableDimensions(gridTableDimensions2,loc2,(int) gridCellSize,5,nLoc2);
	list *tempList;
	int matching[2];
	PossibleKeypointMatchesDims[0] = 12;
	PossibleKeypointMatchesDims[1] = 0;	
	int *gridTable1Loc = new int [gridTableDimensions1[0]*gridTableDimensions1[1]*nLoc1];
	int *gridTable2Loc = new int [gridTableDimensions2[0]*gridTableDimensions2[1]*nLoc2];
	int *NgridTable1Loc = new int [gridTableDimensions1[0]*gridTableDimensions1[1]];
	int *NgridTable2Loc = new int [gridTableDimensions2[0]*gridTableDimensions2[1]];
	int *loc1ToPKM = new int [nLoc1*nLoc2];
	int *loc2ToPKM = new int [nLoc1*nLoc2];
	int *Nloc1ToPKM = new int [nLoc1];
	int *Nloc2ToPKM = new int [nLoc2];
	for (int j=0 ; j<nLoc1; j++)
		Nloc1ToPKM[j] = 0;
	for (int j=0 ; j<nLoc2; j++)
		Nloc2ToPKM[j] = 0;
	divideLocsIntoGrid(gridTable1Loc,NgridTable1Loc,nLoc1,loc1,gridTableDimensions1,gridCellSize);
	divideLocsIntoGrid(gridTable2Loc,NgridTable2Loc,nLoc2,loc2,gridTableDimensions2,gridCellSize);
	for (int matchInd=0; matchInd<nMatches; matchInd++)
	{
		int j, nPoints1 = 0,nPoints2 = 0;		
		while ((inModes1[nPoints1 + (int)(match[0+2*matchInd]-1)*inModes1Dims[0]] > 0) && (nPoints1 < inModes1Dims[0]))
			nPoints1++;
		while ((inModes2[nPoints2 + (int)(match[1+2*matchInd]-1)*inModes2Dims[0]] > 0) && (nPoints2 < inModes2Dims[0]))
			nPoints2++;		
		bool *alreadyTaken1 = new bool[nPoints1];
		bool *alreadyTaken2 = new bool[nPoints2];
		for (j=0; j<nPoints1; j++)
			alreadyTaken1[j] = false;		
		for (int i=0; i<nPoints1; i++)
		{
			if (!alreadyTaken1[i])
			{
				for (j=0; j<nPoints2; j++)
					alreadyTaken2[j] = false;
				list *ind1 = new list;
				tempList = ind1;
				for (j=0; j<nPoints1; j++)
				{
					if ((loc1[0+(int)(inModes1[i+(int)(match[0+2*matchInd]-1)*inModes1Dims[0]]-1)*5] == loc1[0+(int)(inModes1[j+(int)(match[0+2*matchInd]-1)*inModes1Dims[0]]-1)*5] ) && (loc1[1+(int)(inModes1[i+(int)(match[0+2*matchInd]-1)*inModes1Dims[0]]-1)*5] == loc1[1+(int)(inModes1[j+(int)(match[0+2*matchInd]-1)*inModes1Dims[0]]-1)*5] ))
					{
						alreadyTaken1[j] = true;
						(*tempList).data = (int)(inModes1[j+(int)(match[0+2*matchInd]-1)*inModes1Dims[0]]-1);
						(*tempList).next = new list;
						tempList = (*tempList).next;
					}
				}
				(*tempList).data = -1;				
				for (int j=0; j<nPoints2; j++)
				{
					if (!alreadyTaken2[j])
					{
						list *ind2 = new list;
						tempList = ind2;
						for (int k=0; k<nPoints2; k++)
						{
							if ((loc2[0+(int)(inModes2[j+(int)(match[1+2*matchInd]-1)*inModes2Dims[0]]-1)*5] == loc2[0+(int)(inModes2[k+(int)(match[1+2*matchInd]-1)*inModes2Dims[0]]-1)*5]) && (loc2[1+(int)(inModes2[j+(int)(match[1+2*matchInd]-1)*inModes2Dims[0]]-1)*5] == loc2[1+(int)(inModes2[k+(int)(match[1+2*matchInd]-1)*inModes2Dims[0]]-1)*5] ))
							{
								alreadyTaken2[k] = true;
								(*tempList).data = (int)(inModes2[k+(int)(match[1+2*matchInd]-1)*inModes2Dims[0]]-1);
								(*tempList).next = new list;
								tempList = (*tempList).next;
							}
						}
						(*tempList).data = -1;
						if (((*(*ind1).next).data != -1) || ((*(*ind2).next).data != -1))
						{
							getSamePointMatching(matching,ind1,ind2,des1,des2,nLoc1,nLoc2);							
						} else 
						{
							matching[0] = (*ind1).data;
							matching[1] = (*ind2).data;
						}
						for (int k=0; k<5; k++)
						{
							possibleKeypointMatches[k+PossibleKeypointMatchesDims[1]*PossibleKeypointMatchesDims[0]] = loc1[k+matching[0]*5];
							possibleKeypointMatches[k+6+PossibleKeypointMatchesDims[1]*PossibleKeypointMatchesDims[0]] = loc2[k+matching[1]*5];
						}
						possibleKeypointMatches[5+PossibleKeypointMatchesDims[1]*PossibleKeypointMatchesDims[0]] = matchInd+1;
						possibleKeypointMatches[11+PossibleKeypointMatchesDims[1]*PossibleKeypointMatchesDims[0]]= matchInd+1; 
						loc1ToPKM[matching[0]+Nloc1ToPKM[matching[0]]*nLoc1] = PossibleKeypointMatchesDims[1];
						Nloc1ToPKM[matching[0]]++;
						loc2ToPKM[matching[1]+Nloc2ToPKM[matching[1]]*nLoc2] = PossibleKeypointMatchesDims[1];
						Nloc2ToPKM[matching[1]]++;
						PossibleKeypointMatchesDims[1]++;
						delete [] ind2;
					}
				}				
				delete [] ind1;
			}
		}	
		delete [] alreadyTaken1;
		delete [] alreadyTaken2;
	}
	findAllPossible2KeypointMatches(possible2KeypointMatches,possible2KeypointMatchesDescriptors,possibleKeypointMatches,gridTable1Loc,NgridTable1Loc,gridTable2Loc,NgridTable2Loc,loc1,loc2,nLoc1,nLoc2,loc1ToPKM,loc2ToPKM,Nloc1ToPKM,Nloc2ToPKM,gridCellSize,KnnScalesMax,KScales,thresholdSamePoint,knn,gridTableDimensions1,gridTableDimensions2,PossibleKeypointMatchesDims,possible2KeypointMatchesDims,possible2KeypointMatchesDescriptorsDims,match,nMatches,inModes1,inModes2,inModes1Dims,inModes2Dims);
	delete [] gridTable1Loc;
	delete [] gridTable2Loc;
	delete [] NgridTable1Loc;
	delete [] NgridTable2Loc;
	delete [] loc1ToPKM;
	delete [] loc2ToPKM;
	delete [] Nloc1ToPKM;
	delete [] Nloc2ToPKM;
}

void findAllLocsInNeigborhood(int * ind,int * n, double x,double y, double scale, double KnnScalesMax, int gridCellSize, int const * gridTableLoc,int const * NgridTableLoc, int const * dims)
{
	*n = 0;
	int x_ind1 = max(0,(int)((x - scale*KnnScalesMax)/gridCellSize - 0.0000000001));
	int x_ind2 = min(dims[0]-1,(int)((x + scale*KnnScalesMax)/gridCellSize - 0.0000000001));
	int y_ind1 = max(0,(int)((y - scale*KnnScalesMax)/gridCellSize - 0.0000000001));
	int y_ind2 = min(dims[1]-1,(int)((y + scale*KnnScalesMax)/gridCellSize - 0.0000000001));           
	for (int i=x_ind1 ; i<=x_ind2; i++)
		for (int j=y_ind1 ; j<=y_ind2; j++)
			for (int k=0 ; k<NgridTableLoc[i + j*dims[0]]; k++)
			{
				ind[*n] = gridTableLoc[i + j*dims[0] + k*dims[0]*dims[1]];
				(*n)++;
			}
}


bool notSamePoint(int ind1, int ind2, double const * loc)
{
	if ((loc[0+ind1*5] == loc[0+ind2*5]) && (loc[1+ind1*5] == loc[1+ind2*5]) &&	(loc[2+ind1*5] == loc[2+ind2*5]))
		return false;
	else
		return true;
}

void partialSort(int * sortOrder,double const * relativeDist,int * n, int i, int knn, double const * loc, int  nLoc)
{
	if (n[i] > knn)
	{
		double minimumVal;
		int minimumInd,temp,counter = 0,currentI=0;
		while ((currentI < n[i]) && (counter<knn))
		{
			minimumVal = 1e10;
			for (int j=currentI ; j<n[i]; j++)
			{
				if (relativeDist[sortOrder[j+i*nLoc]] < minimumVal)
				{
					minimumVal = relativeDist[sortOrder[j+i*nLoc]];
					minimumInd = j;
				}
			}			
			temp = sortOrder[minimumInd+i*nLoc];
			sortOrder[minimumInd+i*nLoc] = sortOrder[currentI+i*nLoc];
			sortOrder[currentI+i*nLoc] = temp;
			if ((currentI > 0) && (notSamePoint(sortOrder[currentI+i*nLoc],sortOrder[currentI-1+i*nLoc],loc)))
				counter++;
			currentI++;
		}
		n[i] = currentI-1;
	}
}

double myModulus(double angle1,double angle2)
{
	int factor;
	if (angle1 > angle2)
		factor = (int) ((angle1 - angle2)/(2*M_PI));
	else
		factor = (int) ((angle1 - angle2)/(2*M_PI) - 0.9999999999);
	return (angle1 - angle2 - 2*M_PI*factor);
}

double getRelativeAngle(double y,double x,double theta)
{
	return myModulus(atan2(y,x),theta);
}

void getRelativeLoc(double * relativeLoc,int ind1, int ind2,double const * loc)
{
	for (int inner_ind=0 ; inner_ind<2; inner_ind++)
		relativeLoc[inner_ind] = loc[inner_ind+ind2*5] - loc[inner_ind+ind1*5];
}

double getRelativeDistFromLoc(double const * relativeLoc, double scale)
{
	double relativeDist = pow(SQR(relativeLoc[0])+SQR(relativeLoc[1]),0.5)/scale;
	return relativeDist;
}

double getRelativeDist(int ind1, int ind2,double const * loc)
{
	double relativeLoc[2];
	getRelativeLoc(relativeLoc,ind1,ind2,loc);
	double relativeDist = getRelativeDistFromLoc(relativeLoc,loc[2+ind1*5]);
	return relativeDist;
}

void getNnLoc(int * indKnn,int * indKscales,int * nnKnn,int * nnKscales,double const * loc,int i,int nLoc,int const * indLocs,int n, double thresholdSamePoint,int knn,int KnnScalesMax,int KScales)
{
	nnKnn[i] = 0;
	nnKscales[i] = 0;
	double * relativeDist = new double[nLoc];
	for (int j=0 ; j<n; j++)
	{
		relativeDist[indLocs[j]] = getRelativeDist(i,indLocs[j],loc);
		if (relativeDist[indLocs[j]] > thresholdSamePoint/loc[2+i*5]) 
		{
			if (relativeDist[indLocs[j]] < KnnScalesMax)
			{
				indKnn[nnKnn[i]+i*nLoc] = indLocs[j];
				nnKnn[i]++;
			}
			if (relativeDist[indLocs[j]] < KScales)
			{
				indKscales[nnKscales[i]+i*nLoc] = indLocs[j];
				nnKscales[i]++;
			}
		}
	}
	partialSort(indKnn,relativeDist,nnKnn,i,knn,loc,nLoc);
	delete [] relativeDist;
}

double getDistRatio(double dist1,double dist2)
{
	return min(dist1/dist2,dist2/dist1);
}

double getAngleDiff(double angle1,double angle2)
{
	double angleDiff  = myModulus(angle1,angle2);
	return min(angleDiff,2*M_PI-angleDiff);        
}

double getClusterType(double typeMain,double typeNn)
{
	return typeMain == typeNn ? 1 : 0;
}

void findSameF(int * N,double const * possible2KeypointMatches,int const * dims,int offset)
{
	int i,tempInd = 0;
	int * indArray = new int[dims[1]];
	int * NArray   = new int[dims[1]];
	for (int i=0 ; i<dims[1]; i++)	
		indArray[i] = dims[1];
	for (i=0 ; i<dims[1]; i++)
	{
		if (indArray[i] == dims[1])
		{
			indArray[i] = tempInd;
			NArray[tempInd] = 1;
			for (int j=i+1 ; j<dims[1]; j++)
			{
				if ((possible2KeypointMatches[offset+i*dims[0]] == possible2KeypointMatches[offset+j*dims[0]]) && (possible2KeypointMatches[offset+1+i*dims[0]] == possible2KeypointMatches[offset+1+j*dims[0]]) &&
					(possible2KeypointMatches[offset+12+i*dims[0]] == possible2KeypointMatches[offset+12+j*dims[0]]) && (possible2KeypointMatches[offset+13+i*dims[0]] == possible2KeypointMatches[offset+13+j*dims[0]]))
				{	
					indArray[j] = tempInd;
					(NArray[tempInd])++;
				}
			}
			tempInd++;
		}
	}
	for (int i=0 ; i<dims[1]; i++)	
	{
		N[i] = NArray[indArray[i]];
	}
	delete [] indArray;
	delete [] NArray;
}

void createNN(int * indSC, int * indKnn,int * indKscales,int * nnSC,int * nnKnn,int * nnKscales,double const * loc,int nLoc,int const * gridTableLoc,int const * NgridTableLoc,int gridCellSize, int const * gridTableDimensions, double thresholdSamePoint,int knn,int KnnScalesMax,int KScales, int offset, double const * match,int nMatches,double const * inModes, int const * inModesDims)
{
	int ind1,ind2,minimumInd;
	double minimumVal;	
	for (int matchInd=0; matchInd<nMatches; matchInd++)
	{
		int j = 0;
		while (inModes[j + (int)(match[offset+2*matchInd]-1)*inModesDims[0]] > 0)
		{
			int k = 0;
			minimumVal = 1e10;
			ind1 = (int) inModes[j + (int)(match[offset+2*matchInd]-1)*inModesDims[0]] - 1;
			nnSC[ind1] = 0;
			while (inModes[k + (int)(match[offset+2*matchInd]-1)*inModesDims[0]] > 0)
			{
				ind2 = (int) inModes[k + (int)(match[offset+2*matchInd]-1)*inModesDims[0]] - 1;
				double relativeDist = getRelativeDist(ind1,ind2,loc);
				if ((relativeDist < minimumVal) && (relativeDist > thresholdSamePoint/loc[2+ind1*5]))
				{
					minimumVal = relativeDist;
					minimumInd = ind2;
					nnSC[ind1] = 0;
				} else if (relativeDist == minimumVal)
				{
					indSC[nnSC[ind1]+ind1*nLoc] = ind2;
					nnSC[ind1]++;
				}
				k++;
			}
			if (minimumVal != 1e10)
			{
				indSC[nnSC[ind1]+ind1*nLoc] = minimumInd;
				nnSC[ind1]++;
			}
			j++;
		}
	}
	for (int i=0 ; i<nLoc; i++)
	{
		int n;
		int * indLocs = new int[nLoc];
		findAllLocsInNeigborhood(indLocs,&n,loc[0+i*5],loc[1+i*5],loc[2+i*5],KnnScalesMax,gridCellSize,gridTableLoc,NgridTableLoc,gridTableDimensions);			
		getNnLoc(indKnn,indKscales,nnKnn,nnKscales,loc,i,nLoc,indLocs,n,thresholdSamePoint,knn,KnnScalesMax,KScales);
		delete [] indLocs;
	}
}

bool notInList(int ind,int const * allInd,int n)
{
	for (int i=0 ; i<n; i++)
	{
		if (allInd[i] == ind)
			return false;
	}
	return true;
}

int samePKM(int ind1,int ind2,int const * loc1ToPKM,int const * loc2ToPKM,int const * Nloc1ToPKM,int const * Nloc2ToPKM, int nLoc1, int nLoc2)
{
	for (int i=0 ; i<Nloc1ToPKM[ind1]; i++)
	{
		for (int j=0 ; j<Nloc2ToPKM[ind2]; j++)
		{
			if (loc1ToPKM[ind1+i*nLoc1] == loc2ToPKM[ind2+j*nLoc2])
				return loc1ToPKM[ind1+i*nLoc1];
		}
	}
	return -1;
}

void findAllNN(int * nnInd,int * n,int ind1, int ind2,int const * indSC1,int const * indKnn1,int const * indKscales1,int const * indSC2,int const * indKnn2,int const * indKscales2,int const * nnSC1,int const * nnSC2,int const * nnKscales1,int const * nnKscales2,int const * nnKnn1,int const * nnKnn2,int const * loc1ToPKM,int const * loc2ToPKM,int const * Nloc1ToPKM,int const * Nloc2ToPKM,int nLoc1, int nLoc2)
{
	*n = 0;
	for (int j=0 ; j<nnSC1[ind1]; j++)
		for (int k=0 ; k<nnSC2[ind2]; k++)
		{
			nnInd[*n] = samePKM(indSC1[j+ind1*nLoc1],indSC2[k+ind2*nLoc2],loc1ToPKM,loc2ToPKM,Nloc1ToPKM,Nloc2ToPKM,nLoc1,nLoc2);
			if (nnInd[*n] >= 0)
				(*n)++;
		}
	for (int j=0 ; j<nnKnn1[ind1]; j++)
		for (int k=0 ; k<nnKnn2[ind2]; k++)
		{
			nnInd[*n] = samePKM(indKnn1[j+ind1*nLoc1],indKnn2[k+ind2*nLoc2],loc1ToPKM,loc2ToPKM,Nloc1ToPKM,Nloc2ToPKM,nLoc1,nLoc2);
			if ((nnInd[*n] >= 0) && notInList(nnInd[*n],nnInd,*n))
				(*n)++;
		}
	for (int j=0 ; j<nnKscales1[ind1]; j++)
		for (int k=0 ; k<nnKscales2[ind2]; k++)
		{
			nnInd[*n] = samePKM(indKscales1[j+ind1*nLoc1],indKscales2[k+ind2*nLoc2],loc1ToPKM,loc2ToPKM,Nloc1ToPKM,Nloc2ToPKM,nLoc1,nLoc2);
			if ((nnInd[*n] >= 0) && notInList(nnInd[*n],nnInd,*n))
				(*n)++;
		}
}

void findAllPossible2KeypointMatches(double * possible2KeypointMatches, double * possible2KeypointMatchesDescriptors, double const * possibleKeypointMatches,int const * gridTable1Loc,int const * NgridTable1Loc,int const * gridTable2Loc,int const * NgridTable2Loc,double const * loc1, double const * loc2, int nLoc1, int nLoc2,int const * loc1ToPKM,int const * loc2ToPKM,int const * Nloc1ToPKM,int const * Nloc2ToPKM,int gridCellSize, int KnnScalesMax, int KScales, double thresholdSamePoint,int knn, int * gridTableDimensions1,int * gridTableDimensions2, int *possibleKeypointMatchesDims, int *possible2KeypointMatchesDims, int *possible2KeypointMatchesDescriptorsDims,double const * match,int nMatches,double const *inModes1,double const *inModes2,int const * inModes1Dims,int const * inModes2Dims)
{
	int n, ind1,ind2,ind3,ind4;
	double relativeLoc[2];
	int * nnInd = new int[possibleKeypointMatchesDims[1]];	
	int * indSC1 = new int[nLoc1*nLoc1];
	int * indKnn1 = new int[nLoc1*nLoc1];
	int * indKscales1 = new int[nLoc1*nLoc1];
	int * nnSC1 = new int[nLoc1];
	int * nnKnn1 = new int[nLoc1];
	int * nnKscales1 = new int[nLoc1];
	int * indSC2 = new int[nLoc2*nLoc2];
	int * indKnn2 = new int[nLoc2*nLoc2];
	int * indKscales2 = new int[nLoc2*nLoc2];
	int * nnSC2 = new int[nLoc2];
	int * nnKnn2 = new int[nLoc2];
	int * nnKscales2 = new int[nLoc2];
	possible2KeypointMatchesDims[0] = 28;
	possible2KeypointMatchesDims[1] = 0;	
	possible2KeypointMatchesDescriptorsDims[0] = 7;

	createNN(indSC1,indKnn1,indKscales1,nnSC1,nnKnn1,nnKscales1,loc1,nLoc1,gridTable1Loc,NgridTable1Loc,gridCellSize,gridTableDimensions1,thresholdSamePoint,knn,KnnScalesMax,KScales,0,match,nMatches,inModes1,inModes1Dims);
	createNN(indSC2,indKnn2,indKscales2,nnSC2,nnKnn2,nnKscales2,loc2,nLoc2,gridTable2Loc,NgridTable2Loc,gridCellSize,gridTableDimensions2,thresholdSamePoint,knn,KnnScalesMax,KScales,1,match,nMatches,inModes2,inModes2Dims);
	for (int i=0 ; i<possibleKeypointMatchesDims[1]; i++)
	{
		ind1 = (int) possibleKeypointMatches[4+i*possibleKeypointMatchesDims[0]] - 1;
		ind2 = (int) possibleKeypointMatches[10+i*possibleKeypointMatchesDims[0]] - 1;
		findAllNN(nnInd,&n,ind1,ind2,indSC1,indKnn1,indKscales1,indSC2,indKnn2,indKscales2,nnSC1,nnSC2,nnKscales1,nnKscales2,nnKnn1,nnKnn2,loc1ToPKM,loc2ToPKM,Nloc1ToPKM,Nloc2ToPKM,nLoc1,nLoc2);
		for (int j=0 ; j<n; j++)
		{
			for (int internal_ind=0 ; internal_ind<12; internal_ind++)
			{
				possible2KeypointMatches[internal_ind+possible2KeypointMatchesDims[1]*possible2KeypointMatchesDims[0]] = possibleKeypointMatches[internal_ind+i*possibleKeypointMatchesDims[0]];
				possible2KeypointMatches[internal_ind+12+possible2KeypointMatchesDims[1]*possible2KeypointMatchesDims[0]] = possibleKeypointMatches[internal_ind+nnInd[j]*possibleKeypointMatchesDims[0]];						
			}
			ind3 = (int) possibleKeypointMatches[4+nnInd[j]*possibleKeypointMatchesDims[0]] - 1;
			ind4 = (int) possibleKeypointMatches[10+nnInd[j]*possibleKeypointMatchesDims[0]] - 1;			
			getRelativeLoc(relativeLoc,ind1,ind3,loc1);
			possible2KeypointMatches[24+possible2KeypointMatchesDims[1]*possible2KeypointMatchesDims[0]] = getRelativeDistFromLoc(relativeLoc,loc1[2+ind1*5]);
			possible2KeypointMatches[26+possible2KeypointMatchesDims[1]*possible2KeypointMatchesDims[0]] = getRelativeAngle(relativeLoc[1],relativeLoc[0],loc1[3+ind1*5]);
			getRelativeLoc(relativeLoc,ind2,ind4,loc2);
			possible2KeypointMatches[25+possible2KeypointMatchesDims[1]*possible2KeypointMatchesDims[0]] = getRelativeDistFromLoc(relativeLoc,loc2[2+ind2*5]);
			possible2KeypointMatches[27+possible2KeypointMatchesDims[1]*possible2KeypointMatchesDims[0]] = getRelativeAngle(relativeLoc[1],relativeLoc[0],loc2[3+ind2*5]);
			possible2KeypointMatchesDims[1]++;
		}
	}
	possible2KeypointMatchesDescriptorsDims[1] = possible2KeypointMatchesDims[1];
	int * N1 = new int[possible2KeypointMatchesDims[1]];	
	int * N2 = new int[possible2KeypointMatchesDims[1]];	
	findSameF(N1,possible2KeypointMatches,possible2KeypointMatchesDims,0);
	findSameF(N2,possible2KeypointMatches,possible2KeypointMatchesDims,6);
	for (int j=0 ; j<possible2KeypointMatchesDescriptorsDims[1]; j++)
	{		
		possible2KeypointMatchesDescriptors[0+j*7] = N1[j];
		possible2KeypointMatchesDescriptors[1+j*7] = N2[j];
		possible2KeypointMatchesDescriptors[2+j*7] = getDistRatio(possible2KeypointMatches[24+j*possible2KeypointMatchesDims[0]],possible2KeypointMatches[25+j*possible2KeypointMatchesDims[0]]);
		possible2KeypointMatchesDescriptors[3+j*7] = getAngleDiff(possible2KeypointMatches[26+j*possible2KeypointMatchesDims[0]],possible2KeypointMatches[27+j*possible2KeypointMatchesDims[0]]);
		possible2KeypointMatchesDescriptors[4+j*7] = getClusterType(possible2KeypointMatches[5+j*possible2KeypointMatchesDims[0]],possible2KeypointMatches[17+j*possible2KeypointMatchesDims[0]]);
		possible2KeypointMatchesDescriptors[5+j*7] = min(possible2KeypointMatches[24+j*possible2KeypointMatchesDims[0]],possible2KeypointMatches[25+j*possible2KeypointMatchesDims[0]]);
		possible2KeypointMatchesDescriptors[6+j*7] = 0;				
	}
	delete [] N1;
	delete [] N2;
	delete [] nnInd;
	delete [] indSC1;
	delete [] indSC2;
	delete [] indKnn1;
	delete [] indKnn2;
	delete [] indKscales1;
	delete [] indKscales2;
	delete [] nnSC1;
	delete [] nnSC2;
	delete [] nnKnn1;
	delete [] nnKnn2;
	delete [] nnKscales1;
	delete [] nnKscales2;
}               