#include "StdAfx.h"
#include <ctime>
#include <cstdlib>

#define SQR(a)   ((a)*(a))
#define _MATLAB_  

void SampsonDistanceCalculator(double * distances, const double * F,const double * matches,int nMatches);
double * readMat(int *numOfRows, int *numOfCols, char const *fileName);
void printMatToFile(double const * mat, char const *fileName, int nMatches);

#ifndef _MATLAB_
int main(int argc, char* argv[])
{
	/*
		possible arguments:
		1. F file name
		2. matches file name
		3. distances file name
	*/
	char const * argFName = "F.txt";
	char const * argMatchesName = "matches.txt";
	char const * argDistaneName = "distances.txt";

	if (argc >= 4){
		argDistaneName  = argv[3];
	}
	if (argc >= 3){
		argMatchesName  = argv[2];
	}
	if (argc >= 2){
		argFName  = argv[1];
	}
	int nMatches,temp;
	double *F = readMat(&temp,&temp,argFName);
	double *matches = readMat(&nMatches,&temp,argMatchesName);	
	double* distances = new double[nMatches];
	SampsonDistanceCalculator(distances,F,matches,nMatches);
	printMatToFile(distances,argDistaneName,nMatches);	
	delete [] F;
	delete [] matches;
	delete [] distances;
	return 0;
}
#else
#include <matrix.h>
#include <mex.h>   

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{   
	double *F, *matches, *distances;
	const mwSize *dims;

	if(nrhs!=2) 
		mexErrMsgTxt("Two inputs required.");
	if(nlhs!=1) 
		mexErrMsgTxt("One output required.");

	//figure out dimensions
	dims = mxGetDimensions(prhs[1]);

	////associate pointers
	F = mxGetPr(prhs[0]);
	matches = mxGetPr(prhs[1]);
	
	//associate outputs
	plhs[0] = mxCreateDoubleMatrix((int) dims[0],1,mxREAL);
	distances = mxGetPr(plhs[0]);
	SampsonDistanceCalculator(distances,F,matches,(int)dims[0]);
	return;
}
#endif

double * readMat(int *numOfRows, int *numOfCols, char const *fileName)
{
	ifstream file(fileName);
	if (! file){
		cerr << "Could not open file: " << fileName;
		exit(1);
	}	
	file >> *numOfRows;
	file >> *numOfCols;
	double *mat;
	mat = new double[(*numOfRows)*(*numOfCols)];
	for (int j=0 ; j<(*numOfCols) ; j++)
	{
		for (int i=0 ; i<(*numOfRows) ; i++)
		{
			file >> mat[j*(*numOfRows)+i];;
		}
	}
	return mat;
}

void SampsonDistanceCalculator(double * distances,  const double * F,const double * matches,int nMatches)
{
	double v1[3];
	double v2[3];
	
	for (int i=0 ; i<nMatches ; i++)
	{		
		for (int j=0; j<3 ; j++)
		{
			v1[j] = F[j]* matches[i] + F[j+3] * matches[i+nMatches] + F[j+6];
		}
		for (int j=0; j<3 ; j++)
		{
			v2[j] = F[3*j] * matches[i+2*nMatches] + F[1+3*j] * matches[i+3*nMatches] + F[2+3*j];
		}
		distances[i]=(SQR(matches[i+2*nMatches]*v1[0]+matches[i+3*nMatches]*v1[1]+v1[2])) / (SQR(v1[0]) + SQR(v1[1]) + SQR(v2[0]) + SQR(v2[1]));
	}
}

void printMatToFile(double const * mat, char const *fileName, int nMatches)
{
	ofstream file(fileName);
	file << nMatches << endl;
	file << 1 << endl;
	for (int j=0 ; j<1; j++)
	{
		for (int i=0 ; i<nMatches; i++)
		{
			file << mat[i + nMatches*j] << endl;
		}
	}
}
